@@ -38,6 +38,8 @@
 #include "bonobo-activation/bonobo-activation-private.h"
 #include "activation-server-corba-extensions.h"
 
+#include <glib/gstdio.h>
+
 /* no longer used. */
 #define RESIDUAL_SERVERS 0
 
@@ -119,7 +121,7 @@ registry_directory_needs_update (ObjectDirectory *od,
         struct stat statbuf;
         time_t old_mtime;
 
-        if (stat (directory, &statbuf) != 0) {
+        if (g_stat (directory, &statbuf) != 0) {
                 return FALSE;
         }
  
@@ -212,7 +214,7 @@ split_path_unique (const char *colon_delimited_path)
 
         g_return_val_if_fail (colon_delimited_path != NULL, NULL);
 
-        wrk = g_strsplit (colon_delimited_path, ":", -1);
+        wrk = g_strsplit (colon_delimited_path, G_SEARCHPATH_SEPARATOR_S, -1);
 
         g_return_val_if_fail (wrk != NULL, NULL);
 
@@ -331,6 +333,7 @@ od_get_active_server (ObjectDirectory    *od,
 				&servers->servers [i].environment,
 				environment)) {
 			retval = servers->servers [i].server;
+			break;
 		}
         }
 	if (retval != CORBA_OBJECT_NIL &&
@@ -782,6 +785,8 @@ od_register_runtime_server_info (ObjectDirectory  *od,
         GSList *parsed_serverinfo = NULL, *l;
         int     i;
 
+        update_registry (od, FALSE);
+
         old_serverinfo = (Bonobo_ServerInfo *) g_hash_table_lookup (od->by_iid, iid);
         if (old_serverinfo)
                 return old_serverinfo;
