@@ -51,7 +51,8 @@ bonobo_activation_timeout_reg_check_set (gboolean on)
 gboolean
 bonobo_activation_timeout_reg_check (gpointer data)
 {
-        if (!check_registration)
+        if (!check_registration ||
+            g_getenv ("BONOBO_ACTIVATION_NO_TIMEOUT"))
                 return FALSE;
 
         if (need_ior_printout) {
@@ -144,7 +145,7 @@ copy_env_list_to_sequence (Bonobo_ActivationEnvironment *environment,
 			&environment->_buffer [i], val->name, val->value);
 
 #ifdef BONOBO_ACTIVATION_DEBUG
-		g_warning ("Registration environment for '%s' = '%s'%s",
+		g_message ("Registration environment for '%s' = '%s'%s",
 			   environment->_buffer [i].name,
 			   environment->_buffer [i].value,
 			   (environment->_buffer [i].flags &
@@ -153,7 +154,6 @@ copy_env_list_to_sequence (Bonobo_ActivationEnvironment *environment,
 	}
 }
 
-#ifdef BONOBO_ACTIVATION_DEBUG
 static char *
 registration_result_to_string (Bonobo_RegistrationResult result)
 {
@@ -171,13 +171,12 @@ registration_result_to_string (Bonobo_RegistrationResult result)
 		return "(error)";
 		break;
 	default:
-		g_assert_not_reached ();
+                g_warning("Invalid registration result code %i", result);
 		break;
 	}
 
 	return "(invalid)";
 }
-#endif /* BONOBO_ACTIVATION_DEBUG */
 
 /**
  * bonobo_activation_register_active_server:
@@ -273,7 +272,7 @@ bonobo_activation_register_active_server_ext (const char               *iid,
 	CORBA_exception_init (&ev);
 
 #ifdef BONOBO_ACTIVATION_DEBUG
-        g_warning ("About to register '%s': %p%s",
+        g_message ("About to register '%s': %p%s",
                    iid, obj,
                    CORBA_Object_non_existent (obj, &ev) ? " (nonexistent)" : "");
 #endif
@@ -301,20 +300,32 @@ bonobo_activation_register_active_server_ext (const char               *iid,
                          existing, bonobo_activation_client_get (), &ev);
 
                 if (ev._major == CORBA_SYSTEM_EXCEPTION &&
-                    !strcmp (ev._id, ex_CORBA_BAD_OPERATION)) /* fall-back */
+                    !strcmp (ev._id, ex_CORBA_BAD_OPERATION))
+                {
+                            /* fall-back */
+                        CORBA_exception_free (&ev);
                         retval = Bonobo_ObjectDirectory_register_new
                                 (od, iid, reg_env ? &environment : &global_reg_env,
                                  obj, flags, description ? description : "",
                                  existing, &ev);
+                }
+
+                if (ev._major != CORBA_NO_EXCEPTION) {
+                        retval = Bonobo_ACTIVATION_REG_ERROR;
+                        g_warning ("Strange exception (%s) from active server registration",
+                                   ev._id);
+                }
 
 		if (reg_env)
 			CORBA_free (environment._buffer);
         }
 
-#ifdef BONOBO_ACTIVATION_DEBUG
-        g_warning ("registration of '%s' returns %s", iid,
-		   registration_result_to_string (retval));
+#ifndef BONOBO_ACTIVATION_DEBUG
+	if (g_getenv ("BONOBO_DEBUG") && retval != Bonobo_ACTIVATION_REG_SUCCESS)
 #endif
+        	g_message ("registration of '%s' returns %s", iid,
+			   registration_result_to_string (retval));
+
 	if (actid && strcmp (actid, iid) == 0 && need_ior_printout) {
 		char *iorstr;
 		FILE *fh;
@@ -354,7 +365,7 @@ bonobo_activation_register_active_server_ext (const char               *iid,
 	CORBA_exception_free (&ev);
 
 #ifdef BONOBO_ACTIVATION_DEBUG
-        g_warning ("Successfully registered `%s'", iid);
+        g_message ("Successfully registered `%s'", iid);
 #endif
 
 	return retval;
@@ -401,7 +412,7 @@ bonobo_activation_active_server_register (const char   *registration_id,
 					reg_env, "DISPLAY", display_name);
 
 #ifdef BONOBO_ACTIVATION_DEBUG
-		g_warning ("Registering iid '%s' with display '%s'", iid, display_name);
+		g_message ("Registering iid '%s' with display '%s'", iid, display_name);
 #endif
 
 		retval = bonobo_activation_register_active_server (iid, obj, reg_env);
@@ -599,7 +610,7 @@ char *
 bonobo_activation_make_registration_id (const char *iid, const char *display)
 {
 #ifdef BONOBO_ACTIVATION_DEBUG
-        g_warning ("Make registration id from '%s' '%s'", iid, display);
+        g_message ("Make registration id from '%s' '%s'", iid, display);
 #endif
         if (display == NULL) {
                 return g_strdup (iid);
