@@ -16,13 +16,16 @@
 #include <bonobo/bonobo-object.h>
 #include <bonobo/bonobo-context.h>
 #include <bonobo/bonobo-private.h>
+#include <bonobo/bonobo-arg.h>
 
 #include <libintl.h>
 #include <string.h>
 
 #include <glib/gmain.h>
 
+#ifdef HAVE_GTHREADS
 GMutex                   *_bonobo_lock;
+#endif
 CORBA_ORB                 __bonobo_orb = CORBA_OBJECT_NIL;
 PortableServer_POA        __bonobo_poa = CORBA_OBJECT_NIL;
 PortableServer_POAManager __bonobo_poa_manager = CORBA_OBJECT_NIL;
@@ -30,6 +33,100 @@ PortableServer_POAManager __bonobo_poa_manager = CORBA_OBJECT_NIL;
 static guint              bonobo_main_loop_level = 0;
 static GSList *           bonobo_main_loops = NULL;
 
+#ifdef G_OS_WIN32
+
+#ifndef PIC
+#error Must build as a DLL
+#endif
+
+#include <windows.h>
+
+/* Silence gcc with a prototype */
+BOOL WINAPI DllMain (HINSTANCE hinstDLL,
+		     DWORD     fdwReason,
+		     LPVOID    lpvReserved);
+
+static char *bonobo_localedir;
+
+static char *
+replace_prefix (const char *runtime_prefix,
+		const char *configure_time_path)
+{
+	if (strncmp (configure_time_path, PREFIX "/", strlen (PREFIX) + 1) == 0) {
+		return g_strconcat (runtime_prefix,
+				    configure_time_path + strlen (PREFIX),
+				    NULL);
+	} else
+		return g_strdup (configure_time_path);
+}
+
+/* DllMain function needed to fetch the DLL name and deduce the
+ * installation directory from that, and then form the pathname for
+ * bonobo_localedir relative to the installation directory.
+ */
+BOOL WINAPI
+DllMain (HINSTANCE hinstDLL,
+	 DWORD     fdwReason,
+	 LPVOID    lpvReserved)
+{
+	char *prefix = NULL;
+	char *p;
+	
+	switch (fdwReason) {
+	case DLL_PROCESS_ATTACH:
+		if (GetVersion () < 0x80000000) {
+			wchar_t wcbfr[1000];
+
+			/* As bonobo_localedir is passed to the
+			 * non-UTF8ified gettext library, we must have
+			 * it in system codepage. To guard against it
+			 * containing non-representable characters, we
+			 * use the short form of the run-time prefix
+			 * (if available), which is guaranteed to
+			 * contain characters only in the system
+			 * codepage. I think.
+			 */
+			if (GetModuleFileNameW ((HMODULE) hinstDLL,
+						wcbfr, G_N_ELEMENTS (wcbfr)) &&
+			    GetShortPathNameW (wcbfr, wcbfr,
+					       G_N_ELEMENTS (wcbfr)))
+				prefix = g_utf16_to_utf8 (wcbfr, -1,
+							  NULL, NULL, NULL);
+		} else {
+			char cpbfr[1000];
+			if (GetModuleFileNameA ((HMODULE) hinstDLL,
+						cpbfr, G_N_ELEMENTS (cpbfr)))
+				prefix = g_locale_to_utf8 (cpbfr, -1,
+							   NULL, NULL, NULL);
+		}
+
+		if (prefix != NULL) {
+			p = strrchr (prefix, '\\');
+			if (p != NULL)
+				*p = '\0';
+			
+			p = strrchr (prefix, '\\');
+			if (p && (g_ascii_strcasecmp (p + 1, "bin") == 0))
+				*p = '\0';
+		} else {
+			prefix = "";
+		}
+		
+		bonobo_localedir = replace_prefix (prefix, BONOBO_LOCALEDIR);
+		p = bonobo_localedir;
+		bonobo_localedir = g_locale_from_utf8 (bonobo_localedir, -1,
+						       NULL, NULL, NULL);
+		g_free (p);
+		break;
+	}
+	return TRUE;
+}
+
+#undef BONOBO_LOCALEDIR
+#define BONOBO_LOCALEDIR bonobo_localedir
+
+#endif
+
 /**
  * bonobo_orb:
  *
@@ -84,7 +181,7 @@ bonobo_is_initialized (void)
 }
 
 /**
- * bonobo_shutdown:
+ * bonobo_debug_shutdown:
  * 
  *   This shuts down the ORB and any other bonobo related
  * resources.
@@ -161,6 +258,8 @@ bonobo_init_full (int *argc, char **argv,
 	/* Init neccessary bits */
 	g_type_init_with_debug_flags (0);
 
+	bonobo_arg_init ();
+
 	if (!bonobo_activation_is_initialized ())
 		bonobo_activation_init (argc ? *argc : 0, argv);
 
@@ -212,7 +311,9 @@ bonobo_init_full (int *argc, char **argv,
 
 	CORBA_exception_free (&ev);
 
+#ifdef HAVE_GTHREADS
 	_bonobo_lock = g_mutex_new ();
+#endif
 
 	bonobo_context_init ();
 
@@ -300,6 +401,21 @@ bonobo_main (void)
 	bonobo_main_loop_level--;
 }
 
+/**
+ * bonobo_main_level:
+ *
+ * Determines the number of times the bonobo main loop has been entered (minus
+ * the number of exits from the main loop).
+ *
+ * Returns: The number of main loops currently running (0 if no main loops are
+ * running).
+ */
+guint
+bonobo_main_level (void)
+{
+	return bonobo_main_loop_level;
+}
+
 /**
  * bonobo_main_quit:
  * 
